/*----------------------------------------------------------------------- 
 * 
 * File Name: InitSearch.c
 *
 * Author: Brown, D. A. and Creighton, T. D.
 * 
 * Revision: $Id: InitSearch.c,v 1.23 2001/08/27 17:23:23 patrick Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "InitSearch.h"

#ifdef INSPIRAL_SO_ATTACH_DDD
#include <unistd.h>
#endif

NRCSID( INITSEARCHC, PKG "$Id: InitSearch.c,v 1.23 2001/08/27 17:23:23 patrick Exp $" );


void LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams
    )
{
  InspiralSearchParams *params          = NULL;

  INITSTATUS( status, "LALInitSearch", INITSEARCHC );
  ATTATCHSTATUSPTR( status );

  
  /*
   *
   * sleep to give me time to attach ddd
   *
   */


#ifdef INSPIRAL_SO_ATTACH_DDD
  {
   unsigned int doze = 10;
   pid_t myPID;
   myPID = getpid();
   fprintf( stdout, "pid %d sleeping for %d seconds\n", myPID, doze );
   fflush( stdout );
   sleep( doze );
   fprintf( stdout, "pid %d awake\n", myPID );
   fflush( stdout );
  }
#endif


  /*
   *
   * check input, output and parameters
   *
   */


  /* check that the parameter structure exists */
  if ( !initSearchParams )
  { 
    ABORT( status, INITSEARCH_ENULL, INITSEARCH_MSGENULL ); 
  }

  /* and that seachParams is null */
  if ( *searchParams )
  { 
    ABORT( status, INITSEARCH_ENNUL, INITSEARCH_MSGENNUL ); 
  }

  /* check that the arguments exist and there are an acceptable number */
  if ( ! initSearchParams->argv )
  { 
    ABORT( status, INITSEARCH_ENULL, INITSEARCH_MSGENULL ); 
  }
  if ( !( initSearchParams->argc == INSPARG_NBNK 
        || initSearchParams->argc == INSPARG_NCMD 
        || initSearchParams->argc == INSPARG_NILD ) )
  { 
    ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
  }

  /* check that the first argument is -filterparams */
  if ( strcmp( initSearchParams->argv[0], "-filterparams" ) )
  { 
    ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
  }


  /* 
   *
   * sanity check basic arguments
   *
   */


  /* number of points in a data segment */
  if ( atoi( initSearchParams->argv[INSPARG_NPTS] ) <= 0 )
  { 
    ABORT( status, INITSEARCH_ENUMZ, INITSEARCH_MSGENUMZ ); 
  }
  
  /* number of data segments */
  if ( atoi( initSearchParams->argv[INSPARG_NSEG] ) <= 0 )
  { 
    ABORT( status, INITSEARCH_ESEGZ, INITSEARCH_MSGESEGZ ); 
  }
  
  /* data segment overlap */
  if ( atoi( initSearchParams->argv[INSPARG_OVRL] ) < 0 )
  {
    ABORT( status, INITSEARCH_EOVLP, INITSEARCH_MSGEOVLP ); 
  }


  /* 
   *
   * sanity check on filter parameters
   *
   */


  /* low frequency cutoff */
  if ( atof( initSearchParams->argv[INSPARG_FLOW] ) < 1.0 )
  {
    ABORT( status, INITSEARCH_EFLOW, INITSEARCH_MSGEFLOW ); 
  }
  /* inverse spectrum truncation in time domain */
  if ( atoi( initSearchParams->argv[INSPARG_TRNC] ) < 0 )
  {
    ABORT( status, INITSEARCH_ETRNC, INITSEARCH_MSGETRNC );
  }
  /* dynamic range */
  if ( atof( initSearchParams->argv[INSPARG_DYNR] ) < 0 )
  {
    ABORT( status, INITSEARCH_EDYNR, INITSEARCH_MSGEDYNR );
  }
  /* number of chisq bins */
  if ( atoi( initSearchParams->argv[INSPARG_NCHI] ) < 0 )
  { 
    ABORT( status, INITSEARCH_EBINS, INITSEARCH_MSGEBINS ); 
  }


  /* 
   *
   * sanity check on engine parameters
   *
   */


  /* number of templates to exchange */
  if ( atoi( initSearchParams->argv[INSPARG_NTXC] ) <= 0 )
  { 
    ABORT( status, INITSEARCH_ENTXC, INITSEARCH_MSGENTXC ); 
  }

  /* rhosq thresholds: existance */
  if ( ! initSearchParams->argv[INSPARG_RHOT] )
  { 
    ABORT( status, INITSEARCH_ERHOT, INITSEARCH_MSGERHOT ); 
  }

  /* chisq threshold: existance */
  if ( ! initSearchParams->argv[INSPARG_CHIT] )
  {
    ABORT( status, INITSEARCH_ECHIT, INITSEARCH_MSGECHIT );
  }


  /* 
   *
   * sanity check on template input
   *
   */


  /* template source */
  if ( atoi( initSearchParams->argv[INSPARG_TSRC] ) < 0 ||
      atoi( initSearchParams->argv[INSPARG_TSRC] ) > 2  )
  { 
    ABORT( status, INITSEARCH_ETSRC, INITSEARCH_MSGETSRC ); 
  }

  /* template parameters */
  switch ( atoi( initSearchParams->argv[INSPARG_TSRC] ) )
  {
    case 0:
      /* templates from bank: check argc and existence of argument */
      if ( initSearchParams->argc != INSPARG_NBNK )
      {
        ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
      }
      if ( ! initSearchParams->argv[INSPARG_TPRM] )
      {
        ABORT( status, INITSEARCH_ETPRM, INITSEARCH_MSGETPRM ); 
      }
      break;

    case 1:
      /* templates from cmd: check argc and number of templates, templates */
      if ( initSearchParams->argc != INSPARG_NCMD )
      {
        fprintf( stderr, "num args = %d\n", initSearchParams->argc );
        ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
      }
      if ( atoi( initSearchParams->argv[INSPARG_TPRM] ) <= 0 )
      {
        ABORT( status, INITSEARCH_ETMPZ, INITSEARCH_MSGETMPZ );
      }
      if ( !initSearchParams->argv[INSPARG_TMPL] )
      {
        ABORT( status, INITSEARCH_ETPRM, INITSEARCH_MSGETPRM );
      }
      break;

    case 2:
      /* templates from ilwd: check argc */
      if ( initSearchParams->argc != INSPARG_NILD )
      {
        ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
      }
      break;

    default:
      /* should never get here, but just incase */
      ABORT( status, INITSEARCH_ETPRM, INITSEARCH_MSGETPRM );
      break;
  }


  /*
   *
   * allocate memory for the inspiral parameter structure
   *
   */


  *searchParams = params = LALCalloc( 1, sizeof(InspiralSearchParams) ); 
  if ( !params )
  {
    ABORT (status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC);
  }
  

  /*
   *
   * allocate memory for parameters
   *
   */


  /* alloc findchirp initialisation parameters */
  params->initParams = (FindChirpInitParams *) 
    LALCalloc( 1, sizeof(FindChirpInitParams) );

  if ( !params->initParams )
  {
    LALFree (searchParams);
    *searchParams = NULL;
    ABORT (status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC);
  }


  /*
   *
   * assign the initial parameter values
   *
   */


  /* degugging flag */
  params->inspiralDebugFlag = atoi( initSearchParams->argv[INSPARG_CRVC] );

  /* template source */
  params->tmpltSource = atoi( initSearchParams->argv[INSPARG_TSRC] );

  /* search parameters */
  params->searchMaster = initSearchParams->nodeClass;

  /* data segment overlap */
  params->ovrlap = (UINT4) atoi( initSearchParams->argv[INSPARG_OVRL] );

  /* stuff master needs to save between calls */
  params->numTmpltExch = 
    (UINT4) atoi( initSearchParams->argv[INSPARG_NTXC] );
  params->numSlaves     = NULL;
  params->tmpltCurrent  = NULL;

  /* findchirp init parameters */
  params->initParams->numPoints = 
    atoi( initSearchParams->argv[INSPARG_NPTS] );
  
  params->initParams->numSegments = 
    atoi( initSearchParams->argv[INSPARG_NSEG] );
  
  params->numChisqBins = params->initParams->numChisqBins = 
    atoi( initSearchParams->argv[INSPARG_NCHI] );
  
  /* initialize parameter structures */
  params->dataParams   = NULL;
  params->filterParams = NULL;
  params->tmpltParams  = NULL;
  params->fcSegVec     = NULL;
  params->filterInput  = NULL;

  /* make sure the bank head is null */
  params->bankHead     = NULL;


  /* 
   *
   * debugging options 
   *
   */


  /* snrsq output vector */
  if ( params->inspiralDebugFlag == 1 )
  {
    params->initParams->createRhosqVec = 1;
  }
  
  /* record loudest event */
  if ( params->inspiralDebugFlag == 2 )
  {

#ifdef INSPIRAL_SO_DEBUG
    fprintf( stderr, "allocating memory for loudest event\n" );
#endif

    params->loudestEvent = (InspiralEvent *)
      LALCalloc( 1, sizeof(InspiralEvent) );
    if ( ! params->loudestEvent )
    {
      ABORT( status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC );
    }
  }


  /*
   *
   * search master: templates from bank generation routines
   *
   */


  if ( params->searchMaster && params->tmpltSource == 0 )
  {
    INT4                        tmpltFlag;
    InspiralCoarseBankIn        bankIn;
    FindChirpCreateBankParams   createBankParams;

    /* the template parameters should come in the form (0,1,2,3,...)    */
    /* where:                                                           */
    /* (mMin, MMax, mmCoarse, mmFine, fLower, fUpper, tSampling,  REAL8 */
    /* iflso,                                                     INT4  */
    /* numLevel)                                                  UINT4 */

    tmpltFlag = sscanf( initSearchParams->argv[INSPARG_TPRM], 
        "%lf, %lf, %lf, %lf, %lf, %lf, %lf, %d, %u\n",
        &(bankIn.mMin),
        &(bankIn.MMax),
        &(bankIn.mmCoarse),
        &(bankIn.mmFine),
        &(bankIn.fLower),
        &(bankIn.fUpper),
        &(bankIn.tSampling),
        &(bankIn.iflso),
        &(createBankParams.numLevel)
                 ) - 9;

    if ( tmpltFlag ) 
    {
      ABORT( status, INITSEARCH_ETPRM, INITSEARCH_MSGETPRM );
    }

    bankIn.NoisePsd      = LALLIGOIPsd;
    bankIn.method        = one;
    bankIn.order         = twoPN;
    bankIn.approximant   = taylor;
    bankIn.domain        = TimeDomain;
    bankIn.space         = Tau0Tau3;
    bankIn.etamin        = bankIn.mMin * ( bankIn.MMax - bankIn.mMin) /
      ( bankIn.MMax * bankIn.MMax );

    createBankParams.numSegments = params->initParams->numSegments;

    /* call the findchirp routine that wraps the bank generation */
    LALFindChirpCreateInspiralBank( status->statusPtr, &bankIn,
        &(params->bankHead), &createBankParams );
    CHECKSTATUSPTR( status );
  }


  /*
   *
   * search master: templates from command line
   *
   */


  if ( params->searchMaster && params->tmpltSource == 1 )
  {
    InspiralTmpltsFromCmdLine( status->statusPtr, 
        params, initSearchParams->argv );
    CHECKSTATUSPTR( status );
  }


  /* 
   *
   * search slave 
   *
   */


  if ( ! params->searchMaster )
  {
    

    /* 
     *
     * initialize the data conditioning 
     *
     */


    LALFindChirpSPDataInit( status->statusPtr, &(params->dataParams),
        params->initParams );
    CHECKSTATUSPTR( status );

    params->dataParams->invSpecTrunc = 
      atoi( initSearchParams->argv[INSPARG_TRNC] );
    params->dataParams->fLow = 
      (REAL4) atof( initSearchParams->argv[INSPARG_FLOW] );
    params->dataParams->dynRange = 
      pow( 2.0, (REAL4) atof( initSearchParams->argv[INSPARG_DYNR] ) );

    /* allocate memory for the conditioned segments */
    LALCreateFindChirpSegmentVector( status->statusPtr, &(params->fcSegVec),
        params->initParams );
    CHECKSTATUSPTR( status );


    /* 
     *
     * initialize template generation 
     *
     */


    LALFindChirpSPTemplateInit( status->statusPtr, &(params->tmpltParams), 
        params->initParams );
    CHECKSTATUSPTR( status );

    params->tmpltParams->fLow = 
      (REAL4) atof( initSearchParams->argv[INSPARG_FLOW] );
    params->tmpltParams->dynRange = 
      pow( 2.0, (REAL4) atof( initSearchParams->argv[INSPARG_DYNR] ) );


    /* 
     *
     * initialize filter 
     *
     */


    LALCreateFindChirpInput( status->statusPtr, &(params->filterInput), 
        params->initParams );
    CHECKSTATUSPTR( status );
 
    LALFindChirpFilterInit( status->statusPtr, &(params->filterParams), 
        params->initParams );
    CHECKSTATUSPTR( status );

    params->filterParams->computeNegFreq = 0;


    /*
     *
     * initialize chisq veto
     *
     */


    LALFindChirpChisqVetoInit( status->statusPtr, 
        params->filterParams->chisqParams, params->numChisqBins,
        params->initParams->numPoints );
    CHECKSTATUSPTR( status );
    

    /*
     *
     * store thresholds in the inspiral parameter structure
     *
     */


    params->rhosqThreshVec = (REAL4 *)
      LALCalloc( 2, sizeof(REAL4) );
    params->chisqThreshVec = (REAL4 *)
      LALCalloc( 2, sizeof(REAL4) );

    if ( sscanf( initSearchParams->argv[INSPARG_RHOT], "%f, %f\n", 
          params->rhosqThreshVec, params->rhosqThreshVec + 1 ) - 2 )
    {
      ABORT( status, INITSEARCH_ERHOT, INITSEARCH_MSGERHOT ); 
    }

    if ( sscanf( initSearchParams->argv[INSPARG_CHIT], "%f, %f\n", 
          params->chisqThreshVec, params->chisqThreshVec + 1 ) - 2 )
    {
      ABORT( status, INITSEARCH_ECHIT, INITSEARCH_MSGECHIT ); 
    }

  } /* end if master/slave */


  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}


/******** remove all code below here eventually *******************************/


/*
 *
 * Bruce's handy graphing functions
 *
 */


#ifdef INSPIRAL_SO_GRAPHING_FUNCS
/* function to graph an array of REAL4's for debugging */
void
graphREAL4 (
    REAL4      *array, 
    INT4        n,
    INT4        spacing
           ) 
{
  FILE *fp;
  INT4 i;

  /* open a file for writing */
  if ( !(fp = fopen( "temp.graph", "w" )) )
  {
    printf( "couldn't open file\n" );
  }

  /* print data into the file */
  for ( i = 0; i < n; i++ )
    fprintf( fp, "%d\t%e\n", i, array[i * spacing] );

  /* close the file */
  fclose( fp );

  return;
}


/* function to graph an array of INT2's for debugging */
void 
graphINT2 (
    INT2       *array, 
    INT4        n,
    INT4        spacing
          ) 
{
  FILE *fp;
  INT4 i;

  /* open a file for writing */
  if ( !(fp = fopen( "temp.graph", "w" )) )
  {
    printf( "couldn't open file\n" );
  }

  /* print data into the file */
  for ( i = 0; i < n; i++ )
    fprintf( fp, "%d\t%d\n", i, array[i * spacing] );

  /* close the file */
  fclose( fp );

  return;
}


/* function to graph an array of INT4's for debugging */
void 
graphINT4 (
    INT4       *array, 
    INT4        n,
    INT4        spacing
          ) 
{
  FILE *fp;
  INT4 i;

  /* open a file for writing */
  if ( !(fp = fopen( "temp.graph", "w" )) )
  {
    printf( "couldn't open file\n" );
  }

  /* print data into the file */
  for ( i = 0; i < n; i++ )
    fprintf( fp, "%d\t%d\n", i, array[i * spacing] );

  /* close the file */
  fclose( fp );

  return;
}
#endif
