/********************************** <lalVerbatim file="StochasticBarDataCV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: StochasticBarData.c,v 1.3 2001/08/27 18:51:36 heng Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticBarDataC">
\subsection{Module \texttt{StochasticBarData.c}}
\label{s:StochasticBarData.c}

Provides ...

\subsubsection*{Prototypes}
\input{StochasticBarDataCP}
\index{\texttt{LALCreateStochasticDataSegmentVector()}}

\subsection*{Description}

\subsection*{Algorithms}
  
\subsection*{Uses}

\subsection*{Notes}

\vfill{\footnotesize\input{StochasticBarDataCV}}

*************************************************** </lalLaTeX> */

#include <lal/LALStdlib.h>
#include <lal/SeqFactories.h>
#include "StochasticBarData.h"

NRCSID (STOCHASTICDATAC, "$Id: StochasticBarData.c,v 1.3 2001/08/27 18:51:36 heng Exp $");

/************************ <lalVerbatim file="StochasticBarDataCP"> */
void
LALCreateStochasticDataSegmentVector (
    LALStatus*                      status,
    StochasticDataSegmentVector**   vector,
    StochasticInitParams*           params
)
/*********************************************** </lalVerbatim> */
{
    
    INT4                            i;
    StochasticDataSegmentVector*    vectorPtr;
    StochasticDataSegment*          segPtr;

    
    INITSTATUS (status, 
                "LALCreateStochasticDataSegmentVector", 
                STOCHASTICDATAC);
    
    ATTATCHSTATUSPTR (status);


    /* 
     *
     * check output vector and parameters 
     *
     */
    
    ASSERT (!*vector, 
            status, 
            STOCHASTICDATAH_ENNUL, 
            STOCHASTICDATAH_MSGENNUL);

    ASSERT (params, 
            status, 
            STOCHASTICDATAH_ENULL, 
            STOCHASTICDATAH_MSGENULL);
    
    if (params->numSegments <= 0)
    {
        ABORT (status,
               STOCHASTICDATAH_ENSEG, 
               STOCHASTICDATAH_MSGENSEG);
    }
    
    if (params->numPoints <= 0)
    { 
        ABORT (status, 
               STOCHASTICDATAH_ENPTS, 
               STOCHASTICDATAH_MSGENPTS);
    }

    if (params->numFrequencies <= 0)
    {
        ABORT (status, 
               STOCHASTICDATAH_ENFRQ, 
               STOCHASTICDATAH_MSGENFRQ);
    }


    /*
     *
     * allocate memory 
     *
     */
    
    /* pointer to vector */

    vectorPtr = *vector = (StochasticDataSegmentVector *) 
        LALMalloc (sizeof (StochasticDataSegmentVector));
    if (!vector)
    {
        ABORT (status, 
               STOCHASTICDATAH_EALOC, 
               STOCHASTICDATAH_MSGEALOC );
    }
   
    /* assign vector length */ 
    vectorPtr->length = params->numSegments;

    
    /* pointer to segment */

    segPtr = vectorPtr->data = (StochasticDataSegment *) 
        LALMalloc (params->numSegments * sizeof (StochasticDataSegment));
    if (!segPtr)
    {
        /* :TODO: is LALFree() really needed here?? */
        LALFree (vectorPtr);
        vectorPtr = NULL;
        ABORT (status, 
               STOCHASTICDATAH_EALOC, 
               STOCHASTICDATAH_MSGEALOC);
    }

    /* individual data segments */ 

    for (i = 0; i < vectorPtr->length ; ++i)
    {
        /* data1 */
        segPtr[i].data1 = (COMPLEX8TimeSeries *) 
            LALMalloc (sizeof (COMPLEX8TimeSeries));
        if (!(segPtr[i].data1))
        {
            ABORT (status,
                   STOCHASTICDATAH_EALOC, 
                   STOCHASTICDATAH_MSGEALOC);
        }
        
        segPtr[i].data1->data = NULL;
        LALCCreateVector (status->statusPtr, 
                         &(segPtr[i].data1->data), 
                         params->numPoints);
        CHECKSTATUSPTR (status);

        /* data2 */
        segPtr[i].data2 = (COMPLEX8TimeSeries *) 
            LALMalloc (sizeof (COMPLEX8TimeSeries));
        if (!(segPtr[i].data2))
        {
            ABORT (status,
                   STOCHASTICDATAH_EALOC, 
                   STOCHASTICDATAH_MSGEALOC);
        }
        
        segPtr[i].data2->data = NULL;
        LALCCreateVector (status->statusPtr, 
                         &(segPtr[i].data2->data), 
                         params->numPoints);
        CHECKSTATUSPTR (status);
    }

    /* normal exit */
    DETATCHSTATUSPTR (status);
    RETURN (status);
}   


void
LALDestroyStochasticDataSegmentVector(
    LALStatus*                      status,
    StochasticDataSegmentVector**   vector
)
{
    
    INT4                    i;
    StochasticDataSegment*  segPtr;

    
    INITSTATUS (status, 
                "LALDestroyStochasticDataSegmentVector", 
                STOCHASTICDATAC);

    ATTATCHSTATUSPTR (status);

    
    /* 
     *
     * check that there is something that needs to be destroyed 
     *
     */
    
    ASSERT (*vector, 
            status, 
            STOCHASTICDATAH_ENULL, 
            STOCHASTICDATAH_MSGENULL);

    segPtr = (*vector)->data;

    
    /*
     *
     * free memory
     *
     */

    /* :TODO: should all pointers be set to NULL after LALFree?? */
    
    for (i = 0; i < (*vector)->length; ++i)
    {
        LALCDestroyVector (status->statusPtr, 
                          &(segPtr[i].data2->data));
        CHECKSTATUSPTR (status);
        LALFree (segPtr[i].data2);

        LALCDestroyVector (status->statusPtr, 
                          &(segPtr[i].data1->data));
        CHECKSTATUSPTR (status);
        LALFree (segPtr[i].data1);
    }

    LALFree (segPtr);

    LALFree (*vector);
    *vector = NULL;

    /* normal exit */
    DETATCHSTATUSPTR (status);
    RETURN (status);
}

