/********************** <lalVerbatim file="StochasticBuildOutputCV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: StochasticBuildOutput.c,v 1.14 2001/08/27 18:24:33 whelan Exp $
 **************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/Units.h>
#include "StochasticApplySearch.h"

NRCSID (STOCHASTICBUILDOUTPUTC, "power $Id: StochasticBuildOutput.c,v 1.14 2001/08/27 18:24:33 whelan Exp $");

void
StochasticBuildOutput (
      LALStatus*                  status,
      LALSearchOutput*            output,
      COMPLEX8FrequencySeries*    spec,
      gpsTimeInterval             timeInterval
                      )
{

   /*
    * 
    * :TODO: FIRST NEED TO WRITE CC STATISTIC SPECTRUM TO METADATA DATABASE. 
    * :TODO: EVENTUALLY WANT TO WRITE CC STATISTIC SPECTRUM TO TREND FRAMES.
    * 
    * Eventually will need to change this to use optional data field to write
    * frames. For right now, however, we will just write a summ_spectrum
    * table to the database.
    * 
    */


   outPut         *outputArray;
   dataBase       *dbEntry;
   dataBase       *prev;
   UINT4           stringLength;
   multiDimData*   mddEntry;
   CHARVector     *unitString;

   INITSTATUS (status, "StochasticBuildOutput", STOCHASTICBUILDOUTPUTC);
   ATTATCHSTATUSPTR (status);

   output->numOutput = 1;


   /* 
    *
    * create memory for the outPut array
    *
    */

   outputArray = output->result = (outPut *) LALMalloc (sizeof (outPut));
   if (!outputArray)
   {
      ABORT (status, 
            STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC);
   }


   /*
    *
    * fill the array of output
    *
    */


   /* this is the same for every output */
   outputArray->indexNumber = 0;
   outputArray->search         = stocastic;
   /* :TODO: note the incorrect spelling above.  it was entered */
   /* incorrectly in wrapperInterfaceDatatypes.h.  it turns out */
   /* that there is another mispelling in that same file:       */
   /* catagory should spelled category */
   outputArray->significant    = 1;
   outputArray->states         = NULL;
   outputArray->optional       = NULL;


   /* :TODO: the commented out code will become relevant once we are 
    * writing frame files of output spectra, but right now, we use the
    * summ_spectrum table for the results. */
   dbEntry = outputArray->results 
      = (dataBase *) LALMalloc( sizeof(dataBase) );

   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }

   dbEntry->previous = NULL;

   /* start time seconds */
   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit);
   strncpy( dbEntry->columnName, "start_time", dbNameLimit );

   dbEntry->type       = int_4s;
   dbEntry->numberRows = 1;
   dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
   if ( ! dbEntry->rows.int4s )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->rowDimensions = NULL;
   *(dbEntry->rows.int4s) = (INT4) timeInterval.startSec;

   prev = dbEntry;

   /* start time nanoseconds */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "start_time_ns", dbNameLimit );

   dbEntry->type       = int_4s;
   dbEntry->numberRows = 1;
   dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
   if ( ! dbEntry->rows.int4s )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->rowDimensions = NULL;
   *(dbEntry->rows.int4s) = (INT4) timeInterval.startNan;

   prev = dbEntry;

   /* end time seconds */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "end_time", dbNameLimit );

   dbEntry->type       = int_4s;
   dbEntry->numberRows = 1;
   dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
   if ( ! dbEntry->rows.int4s )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->rowDimensions = NULL;
   *(dbEntry->rows.int4s) = (INT4) timeInterval.stopSec;

   prev = dbEntry;

   /* end time nanoseconds */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "end_time_ns", dbNameLimit );

   dbEntry->type       = int_4s;
   dbEntry->numberRows = 1;
   dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
   if ( ! dbEntry->rows.int4s )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->rowDimensions = NULL;
   *(dbEntry->rows.int4s) = (INT4) timeInterval.stopNan;

   prev = dbEntry;

   /* frames used (?) */
   /* :NOTE: we are assuming all frames consist of whole GPS seconds */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "frames_used", dbNameLimit );

   dbEntry->type       = int_4s;
   dbEntry->numberRows = 1;
   dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
   if ( ! dbEntry->rows.int4s )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->rowDimensions = NULL;

   *(dbEntry->rows.int4s)
     = (INT4) timeInterval.stopSec - timeInterval.startSec;

   if ( timeInterval.stopNan != 0 ) *(dbEntry->rows.int4s) += 1;

   prev = dbEntry;

   /* start frequency */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "start_frequency", dbNameLimit );

   dbEntry->type       = real_8;
   dbEntry->numberRows = 1;
   dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
   if ( ! dbEntry->rows.real8 )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->rowDimensions = NULL;
   *(dbEntry->rows.real8) = spec->f0;

   prev = dbEntry;

   /* start frequency */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "delta_frequency", dbNameLimit );

   dbEntry->type       = real_8;
   dbEntry->numberRows = 1;
   dbEntry->rows.real8 = (REAL8 *) LALMalloc( sizeof(REAL8) );
   if ( ! dbEntry->rows.real8 )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->rowDimensions = NULL;
   *(dbEntry->rows.real8) = spec->deltaF;

   prev = dbEntry;

   /* Key for spectrum type. */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "program", dbNameLimit );

   dbEntry->type       = char_s_ptr;
   dbEntry->numberRows = 1;

   /* This prevents extra \0s from being written to the strings */

   stringLength = strlen(STOCHASTICAPPLYSEARCHH_PROGNAME);

   dbEntry->rows.chars = 
      (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
   if ( ! dbEntry->rows.chars )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   strncpy( dbEntry->rows.chars, 
	    STOCHASTICAPPLYSEARCHH_PROGNAME, stringLength*sizeof(CHAR) );

   dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
   if ( ! dbEntry->rowDimensions )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   *(dbEntry->rowDimensions) = stringLength;

   prev = dbEntry;

   /* Key for spectrum type. */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "mimetype", dbNameLimit );

   dbEntry->type       = char_s_ptr;
   dbEntry->numberRows = 1;

   stringLength = strlen(STOCHASTICAPPLYSEARCHH_MIMETYPE);

   dbEntry->rows.chars = 
      (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
   if ( ! dbEntry->rows.chars )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   strncpy( dbEntry->rows.chars, 
	    STOCHASTICAPPLYSEARCHH_MIMETYPE, stringLength*sizeof(CHAR) );

   dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
   if ( ! dbEntry->rowDimensions )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   *(dbEntry->rowDimensions) = stringLength;

   prev = dbEntry;

   /* key for channel name */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "channel", dbNameLimit );

   dbEntry->type       = char_s_ptr;
   dbEntry->numberRows = 1;

   stringLength = strlen(STOCHASTICAPPLYSEARCHH_CHANNEL);

   dbEntry->rows.chars = 
      (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
   if ( ! dbEntry->rows.chars )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   strncpy( dbEntry->rows.chars, STOCHASTICAPPLYSEARCHH_CHANNEL
	    , stringLength*sizeof(CHAR) );

   dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
   if ( ! dbEntry->rowDimensions )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   *(dbEntry->rowDimensions) = stringLength;

   prev = dbEntry;

   /* Key for spectrum type. */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "spectrum_type", dbNameLimit );

   dbEntry->type       = char_s_ptr;
   dbEntry->numberRows = 1;

   stringLength = strlen(STOCHASTICAPPLYSEARCHH_SPECTYPE);

   dbEntry->rows.chars = 
      (CHAR *) LALMalloc( stringLength * dbEntry->numberRows * sizeof(CHAR) );
   if ( ! dbEntry->rows.chars )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   strncpy( dbEntry->rows.chars, 
	     STOCHASTICAPPLYSEARCHH_SPECTYPE, stringLength*sizeof(CHAR) );

   dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
   if ( ! dbEntry->rowDimensions )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   *(dbEntry->rowDimensions) = stringLength;

   prev = dbEntry;

   /* spectrum */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "spectrum", dbNameLimit );

#ifdef STOCHASTIC_SO_SPECTRUM_NOT_BLOB

   dbEntry->type       = complex_8;
   dbEntry->numberRows = spec->data->length;
   dbEntry->rows.complex8 = (COMPLEX8 *) 
            LALMalloc( dbEntry->numberRows * sizeof(COMPLEX8) );
   if ( !(dbEntry->rows.complex8) ) {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
                STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   memcpy( dbEntry->rows.complex8, spec->data->data, dbEntry->numberRows * 
            sizeof(COMPLEX8) );

   dbEntry->rowDimensions = NULL;

#else

   /* Spectrum must be stored as a BLOb */

   dbEntry->type       = char_u_ptr;
   dbEntry->numberRows = 1;
   dbEntry->rowDimensions 
     = (UINT4 *) LALMalloc( dbEntry->numberRows * sizeof(UINT4));
   dbEntry->rowDimensions[0] 
     = spec->data->length * (sizeof(COMPLEX8) / sizeof(UCHAR)) ;
   dbEntry->rows.charu 
     = (UCHAR *) LALMalloc (dbEntry->rowDimensions[0] * sizeof(UCHAR));
   if ( !(dbEntry->rows.charu) ) {
     ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
	    STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }

   /* This is ugly, but should work if there's no padding in storage
    * of COMPLEX8*  */

   memcpy( dbEntry->rows.charu, spec->data->data, 
	   dbEntry->rowDimensions[0] * sizeof(UCHAR) );
#endif

   prev = dbEntry;

   /* spectrum_length */
   dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
   if ( ! dbEntry )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   dbEntry->previous = prev;

   strncpy( dbEntry->tableName, "summ_spectrum", dbNameLimit );
   strncpy( dbEntry->columnName, "spectrum_length", dbNameLimit );

   dbEntry->type       = int_4s;
   dbEntry->numberRows = 1;
   dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
   if ( ! dbEntry->rows.int4s )
   {
      ABORT( status, STOCHASTICAPPLYSEARCHH_EALOC, 
            STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   *(dbEntry->rows.int4s) = spec->data->length;
   dbEntry->rowDimensions = NULL;

   dbEntry->next = NULL;


   /* 
    *
    * alloc memory for the optional linked list
    *
    */
    
    
   /* allocate memory */
   mddEntry = outputArray->optional = (multiDimData *) 
     LALMalloc (sizeof (multiDimData));
   if (!mddEntry)
   {
     ABORT (status, 
	    STOCHASTICAPPLYSEARCHH_EALOC, 
	    STOCHASTICAPPLYSEARCHH_MSGEALOC);
   }
   
   /* write the name and domain of the data */
   strncpy (mddEntry->name, "filter_output", maxMultiDimName);

   /* Read the dimensions from the output frequency series */

   unitString = NULL;
   TRY( LALCHARCreateVector(status->statusPtr, &unitString, LALUnitTextSize),
	status );
   TRY( LALUnitAsString(status->statusPtr, unitString, &(spec->sampleUnits)),
	status );
   strncpy (mddEntry->units, unitString->data, maxMultiDimUnits);
   TRY( LALCHARDestroyVector(status->statusPtr, &unitString),
	status );
   mddEntry->space = freqD;

   mddEntry->range.dFreq.numberSamples = spec->data->length;

   /* printf("%d %d %d %d \n", timeInterval.startSec, timeInterval.startNan,
      timeInterval.stopSec, timeInterval.stopNan); */

   mddEntry->range.dFreq.gpsStartTimeSec = timeInterval.startSec;
   mddEntry->range.dFreq.gpsStartTimeNan = timeInterval.startNan;
   mddEntry->range.dFreq.gpsStopTimeSec = timeInterval.stopSec;
   mddEntry->range.dFreq.gpsStopTimeNan = timeInterval.stopNan;

   /*    printf("%d %d %d %d \n", mddEntry->range.dFreq.gpsStartTimeSec, 
	  mddEntry->range.dFreq.gpsStartTimeNan,
	  mddEntry->range.dFreq.gpsStopTimeSec,
	  mddEntry->range.dFreq.gpsStopTimeNan); */

   mddEntry->range.dFreq.startFreq = spec->f0;
   mddEntry->range.dFreq.stopFreq 
     = spec->f0 + (REAL8)(spec->data->length - 1) * spec->deltaF;
   mddEntry->range.dFreq.freqStepSize = spec->deltaF;
   
   /* data type is COMPLEX8 */
   mddEntry->type = complex_8;
   
   //   /* set the interval union to zero */
   // memset (&(mddEntry->range.dTime), 
   // 0, 
   //   sizeof(gpsTimeInterval));
   
   /* size of the data */
   mddEntry->numberDimensions = 1;
   mddEntry->dimensions = (UINT4 *) LALMalloc (sizeof(UINT4));
   if (!(mddEntry->dimensions))
   {
     ABORT (status, 
	    STOCHASTICAPPLYSEARCHH_EALOC, 
	    STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   mddEntry->dimensions[0] = spec->data->length;
   
   /* dcHistory is meaningless here */
   mddEntry->history = NULL;
   
   /* allocate memory for the data */
   mddEntry->data.complex8 = (COMPLEX8 *) 
     LALCalloc (spec->data->length, sizeof(COMPLEX8));
   if (!(mddEntry->data.complex8))
   {
     ABORT (status, 
	    STOCHASTICAPPLYSEARCHH_EALOC, 
	    STOCHASTICAPPLYSEARCHH_MSGEALOC );
   }
   
   /* and copy the data there */
   memcpy (mddEntry->data.complex8, 
	   spec->data->data, 
	   spec->data->length * sizeof(COMPLEX8) );
   
   /* normal exit */
   DETATCHSTATUSPTR (status);
   RETURN (status);
}
